from gi.repository import Gio, Gtk, GLib, Gdk
from src.gtk.error_toast import ErrorToast
import subprocess, os, pathlib
import gettext

_ = gettext.gettext
home = f"{pathlib.Path.home()}"
icon_theme = Gtk.IconTheme.new()
icon_theme.add_search_path(f"{home}/.local/share/flatpak/exports/share/icons")
direction = Gtk.Image().get_direction()


class Flatpak:
	def open_app(self, callback=None):
		self.failed_app_run = None

		def thread(*args):
			if self.is_runtime:
				self.failed_app_run = "error: cannot open a runtime"
			try:
				subprocess.run(["flatpak", "run", f"{self.info['ref']}"], capture_output=True, text=True, check=True)
			except subprocess.CalledProcessError as cpe:
				self.failed_app_run = cpe
			except Exception as e:
				self.failed_app_run = e

		Gio.Task.new(None, None, callback).run_in_thread(thread)

	def open_data(self):
		if not os.path.exists(self.data_path):
			return f"Path '{self.data_path}' does not exist"
		try:
			Gio.AppInfo.launch_default_for_uri(f"file://{self.data_path}", None)
		except GLib.GError as e:
			return e

	def get_data_size(self, callback=None):
		size = [None]

		def thread(*args):
			sed = "sed 's/K/ KB/; s/M/ MB/; s/G/ GB/; s/T/ TB/; s/P/ PB/;'"
			size[0] = subprocess.run(["sh", "-c", f"du -sh {self.data_path} | {sed}"], capture_output=True, text=True).stdout.split("\t")[0]

		def on_done(*arg):
			if callback:
				callback(f"~ {size[0]}")

		Gio.Task.new(None, None, on_done).run_in_thread(thread)

	def trash_data(self, callback=None):
		try:
			subprocess.run(["gio", "trash", self.data_path], capture_output=True, text=True, check=True)
		except subprocess.CalledProcessError as cpe:
			raise cpe
		except Exception as e:
			raise e

	def set_mask(self, should_mask, callback=None):
		self.failed_mask = None

		def thread(*args):
			cmd = ["flatpak", "mask", self.info["id"]]
			installation = self.info["installation"]
			if installation == "user" or installation == "system":
				cmd.append(f"--{installation}")
			else:
				cmd.append(f"--installation={installation}")

			if not should_mask:
				cmd.append("--remove")

			try:
				subprocess.run(cmd, check=True, capture_output=True, text=True)
				self.is_masked = should_mask
			except subprocess.CalledProcessError as cpe:
				self.failed_mask = cpe
			except Exception as e:
				self.failed_mask = e

		Gio.Task.new(None, None, callback).run_in_thread(thread)

	def set_pin(self, should_pin, callback=None):
		self.failed_pin = None
		if not self.is_runtime:
			self.failed_pin = "Cannot pin an application"

		def thread(*args):
			cmd = ["flatpak", "pin", f"runtime/{self.info['ref']}"]
			installation = self.info["installation"]
			if installation == "user" or installation == "system":
				cmd.append(f"--{installation}")
			else:
				cmd.append(f"--installation={installation}")

			if not should_pin:
				cmd.append("--remove")

			try:
				subprocess.run(cmd, check=True, capture_output=True, text=True)
			except subprocess.CalledProcessError as cpe:
				self.failed_pin = cpe
			except Exception as e:
				self.failed_mask = e

		Gio.Task.new(None, None, callback).run_in_thread(thread)

	def uninstall(self, callee_callback=None):
		self.failed_uninstall = None

		def callback(*args):
			HostInfo.main_window.remove_refresh_lockout("uninstalling packages")
			if not callee_callback is None:
				callee_callback()

		def thread(*args):
			HostInfo.main_window.add_refresh_lockout("uninstalling packages")
			cmd = ["flatpak", "uninstall", "-y", self.info["ref"]]
			installation = self.info["installation"]
			if installation == "system" or installation == "user":
				cmd.append(f"--{installation}")
			else:
				cmd.append(f"--installation={installation}")

			try:
				subprocess.run(cmd, check=True, text=True, capture_output=True)
			except subprocess.CalledProcessError as cpe:
				self.failed_uninstall = cpe
			except Exception as e:
				self.failed_uninstall = e

		Gio.Task.new(None, None, callback).run_in_thread(thread)

	def get_cli_info(self):
		cli_info = {}
		cmd = "LC_ALL=C flatpak info "
		installation = self.info["installation"]

		if installation == "user":
			cmd += "--user "
		elif installation == "system":
			cmd += "--system "
		else:
			cmd += f"--installation={installation} "

		cmd += self.info["ref"]
		try:
			output = subprocess.run(["sh", "-c", cmd], text=True, capture_output=True).stdout
		except Exception as e:
			raise e

		lines = output.strip().split("\n")
		cli_info["description"] = ""
		first = lines.pop(0)
		if " - " in first:
			cli_info["description"] = first.split(" - ")[1]

		# Handle descriptions that contain newlines
		while (line := lines.pop(0)) and not ":" in line:
			if len(line) > 0:
				cli_info["description"] += f" {line}"

		for i, word in enumerate(lines):
			if not ":" in word:
				continue

			word = word.strip().split(": ", 1)
			if len(word) < 2:
				continue

			word[0] = word[0].lower()
			if "installed" in word[0]:
				word[1] = word[1].replace("?", " ")
			cli_info[word[0]] = word[1]

		self.cli_info = cli_info
		return cli_info

	def __init__(self, columns):
		self.info = {
			"name": columns[0],
			"id": columns[1],
			"version": columns[2],
			"branch": columns[3],
			"arch": columns[4],
			"origin": columns[5],
			"installation": columns[6],
			"ref": columns[7],
			"installed_size": columns[8],
			"options": columns[9],
		}
		self.is_runtime = "runtime" in self.info["options"]
		self.data_path = f"{home}/.var/app/{self.info["id"]}"
		self.data_size = -1
		self.cli_info = None
		installation = self.info["installation"]
		if len(i := installation.split(" ")) > 1:
			self.info["installation"] = i[1].replace("(", "").replace(")", "")
		else:
			self.info["installation"] = installation

		self.is_eol = "eol=" in self.info["options"]
		self.dependent_runtime = None
		self.failed_app_run = None
		self.failed_mask = None
		self.failed_uninstall = None
		self.app_row = None

		try:
			self.is_masked = self.info["id"] in HostInfo.masks[self.info["installation"]]
		except KeyError:
			self.is_masked = False

		try:
			self.is_pinned = f"runtime/{self.info['ref']}" in HostInfo.pins[self.info["installation"]]
		except KeyError:
			self.is_pinned = False

		try:
			self.icon_path = icon_theme.lookup_icon(self.info["id"], None, 512, 1, direction, 0).get_file().get_path()
		except GLib.GError as e:
			print(f"Minor error in looking up icon for {self.info['id']}", e)
			self.icon_path = None


class Remote:
	def __init__(self, name, title, disabled):
		self.name = name
		self.title = title
		self.disabled = disabled
		if title == "" or title == "-":
			self.title = name


class HostInfo:
	home = home
	clipboard = Gdk.Display.get_default().get_clipboard()
	main_window = None
	snapshots_path = f"{home}/.var/app/io.github.flattool.Warehouse/data/Snapshots/"

	# Get all possible installation icon theme dirs
	output = subprocess.run(
		["flatpak", "--installations"],
		text=True,
		capture_output=True,
	).stdout
	lines = output.strip().split("\n")
	for i in lines:
		icon_theme.add_search_path(f"{i}/exports/share/icons")

	flatpaks = []
	id_to_flatpak = {}
	ref_to_flatpak = {}
	remotes = {}
	installations = []
	masks = {}
	pins = {}
	dependent_runtime_refs = []

	@classmethod
	def get_flatpaks(this, callback=None):
		# Callback is a function to run after the host flatpaks are found
		this.flatpaks.clear()
		this.id_to_flatpak.clear()
		this.ref_to_flatpak.clear()
		this.remotes.clear()
		this.installations.clear()
		this.masks.clear()
		this.pins.clear()
		this.dependent_runtime_refs.clear()

		def thread(task, *args):
			# Remotes
			def remote_info(installation):
				cmd = ["flatpak", "remotes", "--columns=name,title,options", "--show-disabled"]
				if installation == "user" or installation == "system":
					cmd.append(f"--{installation}")
				else:
					cmd.append(f"--installation={installation}")
				output = subprocess.run(
					cmd,
					text=True,
					capture_output=True,
				).stdout
				lines = output.strip().split("\n")
				remote_list = []
				if lines[0] != "":
					for line in lines:
						line = line.split("\t")
						remote_list.append(Remote(name=line[0], title=line[1], disabled=(len(line) == 3) and "disabled" in line[2]))
				this.remotes[installation] = remote_list

				# Masks
				cmd = [
					"flatpak",
					"mask",
				]
				if installation == "user" or installation == "system":
					cmd.append(f"--{installation}")
				else:
					cmd.append(f"--installation={installation}")
				output = subprocess.run(
					cmd,
					text=True,
					capture_output=True,
				).stdout
				lines = output.strip().replace(" ", "").split("\n")
				if lines[0] != "":
					this.masks[installation] = lines

				# Pins
				cmd = [
					"flatpak",
					"pin",
				]
				if installation == "user" or installation == "system":
					cmd.append(f"--{installation}")
				else:
					cmd.append(f"--installation={installation}")
				output = subprocess.run(
					cmd,
					text=True,
					capture_output=True,
				).stdout
				lines = output.strip().replace(" ", "").split("\n")
				if lines[0] != "":
					this.pins[installation] = lines

			try:
				# Installations
				# Get all config files for any extra installations
				custom_install_config_path = "/run/host/etc/flatpak/installations.d"
				if os.path.exists(custom_install_config_path):
					for file in os.listdir(custom_install_config_path):
						with open(f"{custom_install_config_path}/{file}", "r") as f:
							for line in f:
								if line.startswith("[Installation"):
									# Get specifically the installation name itself
									this.installations.append(line.replace('[Installation "', "").replace('"]', "").strip())

				this.installations.append("user")
				this.installations.append("system")
				for i in this.installations:
					remote_info(i)

				# Packages
				output = subprocess.run(
					["flatpak", "list", "--columns=name,application,version,branch,arch,origin,installation,ref,size,options"],
					text=True,
					check=True,
					capture_output=True,
				).stdout
				lines = output.strip().split("\n")
				for i in lines:
					package = Flatpak(i.split("\t"))
					this.flatpaks.append(package)
					this.id_to_flatpak[package.info["id"]] = package
					this.ref_to_flatpak[package.info["ref"]] = package

				# Dependent Runtimes
				output = subprocess.run(
					["flatpak", "list", "--columns=runtime,ref"],
					text=True,
					check=True,
					capture_output=True,
				).stdout
				lines = output.split("\n")
				for index, line in enumerate(lines):
					split_line = line.split("\t")
					if len(split_line) < 2 or split_line[0] == "":
						continue

					package = this.flatpaks[index]
					if package.is_runtime:
						continue

					runtime = split_line[0]
					package.dependent_runtime = this.ref_to_flatpak[runtime]
					if not runtime in this.dependent_runtime_refs:
						this.dependent_runtime_refs.append(runtime)

				# store runtimes in sorted order
				this.dependent_runtime_refs = sorted(this.dependent_runtime_refs)

			except subprocess.CalledProcessError as cpe:
				this.main_window.toast_overlay.add_toast(ErrorToast(_("Could not load packages"), cpe.stderr).toast)
			except Exception as e:
				this.main_window.toast_overlay.add_toast(ErrorToast(_("Could not load packages"), str(e)).toast)

		Gio.Task.new(None, None, callback).run_in_thread(thread)



from gi.repository import Adw, Gtk, GLib, Gio
from src.host_info import HostInfo
from src.install_page.result_row import ResultRow
from src.gtk.loading_status import LoadingStatus
from src.gtk.error_toast import ErrorToast
import subprocess
import gettext

_ = gettext.gettext


class AddedPackage:
	def __eq__(self, other):
		return (
			self.name == other.name
			and self.app_id == other.app_id
			and self.branch == other.branch
			and self.version == other.version
			and self.remote == other.remote
			and self.installation == other.installation
		)

	def is_similar(self, other):
		return self.app_id == other.app_id and self.branch == other.branch and self.version == other.version

	def __init__(self, name, app_id, branch, version, remote, installation):
		self.name = name
		self.app_id = app_id
		self.branch = branch
		self.version = version
		self.remote = remote
		self.installation = installation


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/install_page/results_page.ui")
class ResultsPage(Adw.NavigationPage):
	__gtype_name__ = "ResultsPage"
	gtc = Gtk.Template.Child

	search_entry = gtc()
	results_list = gtc()
	stack = gtc()
	new_search = gtc()
	too_many = gtc()
	results_view = gtc()
	no_results = gtc()

	def show_remote(self, row, remote, installation, nav_view=None):
		self.remote = remote
		self.installation = installation
		self.set_title(_("Search {}").format(remote.title))
		self.search_entry.set_text("")
		self.search_entry.grab_focus()
		if nav_view:
			nav_view.push(self)

	def add_package_row(self, row):
		self.pending_page.add_package_row(row)
		if not self.install_page is None:
			self.install_page.package_added()

	def on_search(self, *args):
		self.packages.clear()
		self.stack.set_visible_child(self.loading)
		self.results_list.remove_all()
		search_text = self.search_entry.get_text()
		if search_text == "":
			self.stack.set_visible_child(self.new_search)
			return

		def thread(*args):
			installation = ""
			if self.installation == "user" or self.installation == "system":
				installation = f"--{self.installation}"
			else:
				installation = f"--installation={self.installation}"

			try:
				output = subprocess.run(
					["flatpak", "search", "--columns=all", installation, self.search_entry.get_text()],
					check=True,
					text=True,
					capture_output=True,
				).stdout.split("\n")
				if len(output) > 100:
					GLib.idle_add(lambda *_: self.stack.set_visible_child(self.too_many))
					return

				for line in output:
					line = line.strip()

					info = line.split("\t")
					if len(info) != 6:
						continue

					remotes = info[5].split(",")
					if not self.remote.name in remotes:
						continue

					package = AddedPackage(info[0], info[2], info[4], info[3], self.remote, self.installation)
					row = ResultRow(package, ResultRow.PackageState.NEW, self.results_list)
					for item in self.pending_page.added_packages:
						if package.is_similar(item):
							row.set_state(ResultRow.PackageState.SELECTED)

					if package.app_id in HostInfo.id_to_flatpak:
						installed_package = HostInfo.id_to_flatpak[package.app_id]
						if installed_package.info["id"] == package.app_id and installed_package.info["branch"] == package.branch:
							row.set_state(ResultRow.PackageState.INSTALLED)

					row.connect("activated", self.add_package_row)
					self.packages.append(package)
					GLib.idle_add(lambda *_, _row=row: self.results_list.append(_row))

				if len(self.packages) > 0:
					GLib.idle_add(lambda *_: self.stack.set_visible_child(self.results_view))
				else:
					GLib.idle_add(lambda *_: self.stack.set_visible_child(self.no_results))

			except subprocess.CalledProcessError as cpe:
				GLib.idle_add(lambda *_, cpe=cpe: HostInfo.main_window.toast_overlay.add_toast(ErrorToast("Could not search for package", cpe.stderr).toast))
				GLib.idle_add(lambda *_: self.install_page.select_page.nav_view.pop())

		Gio.Task().run_in_thread(thread)

	def on_back(self, *args):
		self.results_list.remove_all()
		self.stack.set_visible_child(self.new_search)

	def __init__(self, **kwargs):
		super().__init__(**kwargs)

		# Extra Object Creation
		self.remote = None
		self.installation = None
		self.packages = []
		self.pending_page = None
		self.loading = LoadingStatus(_("Searching"), _("This should only take a moment"))
		self.install_page = None

		# Connections
		self.search_entry.connect("search-changed", self.on_search)

		# Apply
		self.stack.add_child(self.loading)



from gi.repository import Gtk
from src.host_info import HostInfo
import gettext

_ = gettext.gettext


class SidebarButton(Gtk.Button):
	__gtype_name__ = "SidebarButton"

	def __init__(self, **kwargs):
		super().__init__(**kwargs)

		# Extra Object Creation
		main_split = HostInfo.main_window.main_split

		# Connections
		main_split.connect("notify::collapsed", lambda *_: self.set_visible(main_split.get_collapsed()))
		self.connect("clicked", lambda *_: main_split.set_show_sidebar(True))

		# Apply
		self.set_icon_name("warehouse:dock-left-symbolic")
		self.set_tooltip_text(_("Show Sidebar"))
		self.set_visible(False)



from gi.repository import Adw, Gtk, GLib
from src.host_info import HostInfo
from src.gtk.loading_status import LoadingStatus
from src.gtk.app_row import AppRow
from src.snapshot_page.tar_worker import TarWorker
import os, time
import gettext

_ = gettext.gettext


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/snapshot_page/new_snapshot_dialog.ui")
class NewSnapshotDialog(Adw.Dialog):
	__gtype_name__ = "NewSnapshotDialog"
	gtc = Gtk.Template.Child

	toast_overlay = gtc()
	nav_page = gtc()
	list_cancel_button = gtc()
	search_button = gtc()
	create_button = gtc()
	search_entry = gtc()
	name_entry = gtc()
	listbox = gtc()
	select_all_button = gtc()
	total_selected_label = gtc()
	scrolled_window = gtc()
	no_results = gtc()
	stack = gtc()
	is_open = False

	def row_gesture_handler(self, row):
		row.check_button.set_active(not row.check_button.get_active())

	def row_select_handler(self, row):
		if row.check_button.get_active():
			self.selected_rows.append(row)
		else:
			self.selected_rows.remove(row)

		total = len(self.selected_rows)
		self.total_selected_label.set_label(_("{} Selected").format(total))
		self.total_selected_label.set_visible(total > 0)
		self.valid_checker()

	def generate_list(self, *args):
		for package in HostInfo.flatpaks:
			if "io.github.flattool.Warehouse" in package.info["id"]:
				continue

			if package.is_runtime or not os.path.exists(package.data_path):
				continue

			row = AppRow(package, self.row_gesture_handler)
			row.check_button.set_visible(True)
			row.check_button.connect("toggled", lambda *_, row=row: self.row_select_handler(row))
			row.set_activatable(True)
			row.set_activatable_widget(row.check_button)
			self.listbox.append(row)

	def sort_func(self, row1, row2):
		return row1.package.info["name"].lower() > row2.package.info["name"].lower()

	def filter_func(self, row):
		title = row.get_title().lower()
		subtitle = row.get_subtitle().lower()
		search = self.search_entry.get_text().lower()
		if search in title or search in subtitle:
			self.is_result = True
			return True
		else:
			return False

	def on_close(self, *args):
		self.__class__.is_open = False
		self.search_button.set_active(False)
		for row in self.selected_rows.copy():
			GLib.idle_add(lambda *_, row=row: row.check_button.set_active(False))

	def valid_checker(self):
		text = self.name_entry.get_text().strip()
		something_selected = len(self.selected_rows) > 0
		text_good = len(text) > 0 and not ("/" in text or "\0" in text)
		self.create_button.set_sensitive(something_selected and text_good)
		if text_good:
			self.name_entry.remove_css_class("error")
		else:
			self.name_entry.add_css_class("error")

		return something_selected and text_good

	def get_total_fraction(self):
		total = 0
		stopped_workers_amount = 0
		for worker in self.workers:
			total += worker.fraction
			if worker.stop:
				stopped_workers_amount += 1

			if stopped_workers_amount == len(self.workers):
				self.loading_status.progress_bar.set_fraction(1)
				self.loading_status.progress_label.set_label(f"{len(self.workers)} / {len(self.workers)}")
				self.workers.clear()
				if self.on_done:
					self.on_done()

				return False

		self.loading_status.progress_label.set_label(f"{stopped_workers_amount + 1} / {len(self.workers)}")
		self.loading_status.progress_bar.set_fraction(total / len(self.workers))
		return True

	def on_create(self, button):
		self.loading_status.title_label.set_label(_("Creating Snapshot"))
		self.loading_status.progress_bar.set_fraction(0.0)
		self.snapshot_page.status_stack.set_visible_child(self.snapshot_page.snapshotting_view)
		self.workers.clear()
		for row in self.selected_rows:
			if "io.github.flattool.Warehouse" in row.package.info["id"]:
				continue

			package = row.package
			worker = TarWorker(
				existing_path=package.data_path,
				new_path=f"{HostInfo.snapshots_path}{package.info['id']}",
				file_name=f"{int(time.time())}_{package.info["version"]}",
				name=self.name_entry.get_text(),
				toast_overlay=self.snapshot_page.toast_overlay,
			)
			self.workers.append(worker)
			worker.compress()

		self.loading_status.progress_label.set_visible(len(self.workers) > 1)
		GLib.timeout_add(200, self.get_total_fraction)
		self.close()

	def on_invalidate(self, search_entry):
		self.is_result = False
		self.listbox.invalidate_filter()
		if self.is_result:
			self.stack.set_visible_child(self.scrolled_window)
		else:
			self.stack.set_visible_child(self.no_results)

	def on_select_all(self, button):
		i = 0
		while row := self.listbox.get_row_at_index(i):
			i += 1
			row.check_button.set_active(True)

	def set_packages(self):
		for package in self.packages:
			row = AppRow(package)
			row.set_activatable(False)
			self.selected_rows.append(row)
			self.listbox.append(row)

	def enter_handler(self, *args):
		if self.create_button.get_sensitive():
			self.create_button.activate()

	def present(self, *args, **kwargs):
		if self.__class__.is_open:
			return

		super().present(*args, **kwargs)
		self.__class__.is_open = True
		if not self.search_button.get_visible():
			self.name_entry.grab_focus()

	def __init__(self, snapshot_page, loading_status, on_done=None, packages=None, **kwargs):
		super().__init__(**kwargs)

		# Extra Object Creations
		self.snapshot_page = snapshot_page
		self.loading_status = loading_status
		self.on_done = on_done
		self.is_result = False
		self.rows = []
		self.selected_rows = []
		self.workers = []
		self.packages = packages

		# Connections
		self.connect("closed", self.on_close)
		self.create_button.connect("clicked", self.on_create)
		self.search_entry.connect("search-changed", self.on_invalidate)
		self.list_cancel_button.connect("clicked", lambda *_: self.close())
		self.name_entry.connect("changed", lambda *_: self.valid_checker())
		self.name_entry.connect("entry-activated", self.enter_handler)
		self.select_all_button.connect("clicked", self.on_select_all)

		# Apply
		self.listbox.set_sort_func(self.sort_func)
		self.listbox.set_filter_func(self.filter_func)
		self.name_entry.set_title(_("Name these Snapshots"))
		if not packages is None:
			self.search_entry.set_editable(False)
			self.search_button.set_visible(False)
			self.nav_page.set_title(_("New Snapshot"))
			self.set_packages()
			self.no_results.set_visible(False)
			if len(packages) == 1:
				self.name_entry.set_title(_("Name this Snapshot"))
		else:
			self.nav_page.set_title(_("New Snapshots"))
			self.generate_list()



from gi.repository import Adw, Gtk, GLib, Gio
from src.host_info import HostInfo
from src.gtk.error_toast import ErrorToast
from src.install_page.results_page import ResultsPage
from src.gtk.sidebar_button import SidebarButton
from src.install_page.file_install_dialog import FileInstallDialog
import gettext

_ = gettext.gettext


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/install_page/select_page.ui")
class SelectPage(Adw.NavigationPage):
	__gtype_name__ = "SelectPage"
	gtc = Gtk.Template.Child

	nav_view = gtc()
	results_page = gtc()
	remotes_group = gtc()
	add_remote_row = gtc()
	open_row = gtc()

	def start_loading(self):
		self.nav_view.pop()
		for row in self.remote_rows:
			self.remotes_group.remove(row)
		self.remote_rows.clear()

	def end_loading(self):
		for installation, remotes in HostInfo.remotes.items():
			for remote in remotes:
				if remote.disabled:
					continue

				row = Adw.ActionRow(title=remote.title, subtitle=_("Installation: {}").format(installation), activatable=True)
				row.add_suffix(Gtk.Image(icon_name="warehouse:right-large-symbolic"))
				row.connect("activated", self.results_page.show_remote, remote, installation, self.nav_view)
				self.remotes_group.add(row)
				self.remote_rows.append(row)

		self.remotes_group.set_visible(len(self.remote_rows) != 0)

	def local_install_apply_callback(self, installation, file_names):
		install_page = HostInfo.main_window.pages[HostInfo.main_window.install_row]
		requests = []
		for file in file_names:
			# sadly flatpak doesn't support multiple local installs in one command :(
			requests.append(
				{
					"remote": "local_file",
					"installation": installation,
					"package_names": [file.get_path()],
					"extra_flags": [],
				}
			)

		install_page.install_packages(requests)

	def file_dialog_handler(self, files):
		FileInstallDialog(self, files, self.local_install_apply_callback).present(HostInfo.main_window)

	def file_choose_callback(self, object, result):
		try:
			files = object.open_multiple_finish(result)
			if not files:
				HostInfo.main_window.toast_overlay.add_toast(ErrorToast(_("Could not add files"), _("No files were found to install")))
				return

			self.file_dialog_handler(files)

		except GLib.GError as gle:
			if not (gle.domain == "gtk-dialog-error-quark" and gle.code == 2):
				HostInfo.main_window.toast_overlay.add_toast(ErrorToast(_("Could not add files"), str(gle)).toast)

	def on_open(self, *args):
		file_filter = Gtk.FileFilter(name=_("Flatpaks"))
		file_filter.add_suffix("flatpak")
		file_filter.add_suffix("flatpakref")
		filters = Gio.ListStore.new(Gtk.FileFilter)
		filters.append(file_filter)
		file_chooser = Gtk.FileDialog()
		file_chooser.set_filters(filters)
		file_chooser.set_default_filter(file_filter)
		file_chooser.open_multiple(HostInfo.main_window, None, self.file_choose_callback)

	def __init__(self, **kwargs):
		super().__init__(**kwargs)

		# Extra Object Creation
		self.remote_rows = []

		# Connections
		self.add_remote_row.connect("activated", lambda *_: HostInfo.main_window.activate_row(HostInfo.main_window.remotes_row))
		self.nav_view.connect("popped", self.results_page.on_back)
		self.open_row.connect("activated", self.on_open)



from gi.repository import Adw, Gtk, GLib, Gio
from src.gtk.error_toast import ErrorToast
from src.host_info import HostInfo
from src.change_version_page.change_version_page import ChangeVersionPage
from src.packages_page.uninstall_dialog import UninstallDialog
from src.gtk.loading_status import LoadingStatus
from src.package_install_worker import PackageInstallWorker
from src.const import Config
import subprocess, os
import gettext

_ = gettext.gettext


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/properties_page/properties_page.ui")
class PropertiesPage(Adw.NavigationPage):
	__gtype_name__ = "PropertiesPage"
	gtc = Gtk.Template.Child
	stack = gtc()
	error_tbv = gtc()
	loading_tbv = gtc()

	more_menu = gtc()
	more_list = gtc()

	nav_view = gtc()
	inner_nav_page = gtc()
	toast_overlay = gtc()
	header_bar = gtc()
	scrolled_window = gtc()
	app_icon = gtc()
	name = gtc()
	description = gtc()
	eol_box = gtc()
	open_app_button = gtc()
	uninstall_button = gtc()

	pin_row = gtc()
	pin_switch = gtc()
	data_row = gtc()
	open_data_button = gtc()
	trash_data_button = gtc()
	data_spinner = gtc()
	version_row = gtc()
	mask_label = gtc()
	mask_row = gtc()
	mask_switch = gtc()
	change_version_row = gtc()
	installed_size_row = gtc()
	runtime_row = gtc()
	eol_package_package_status_icon = gtc()

	id_row = gtc()
	ref_row = gtc()
	arch_row = gtc()
	branch_row = gtc()
	license_row = gtc()

	sdk_row = gtc()
	origin_row = gtc()
	collection_row = gtc()
	installation_row = gtc()

	commit_row = gtc()
	parent_row = gtc()
	subject_row = gtc()
	date_row = gtc()

	package = None

	def set_properties(self, package, refresh=False):
		if package == self.package and not refresh:
			# Do not update the ui if the same app row is clicked
			return

		self.reinstall_did_error = False
		self.package = package
		package_is_not_this_app = package.info["id"] != Config.APP_ID

		# Show Package Name
		pkg_name = package.info["name"]
		if pkg_name != "":
			self.inner_nav_page.set_title(_("{} Properties").format(pkg_name))
			self.name.set_label(pkg_name)
			self.name.set_visible(True)
		else:
			self.inner_nav_page.set_title(_("Properties"))
			self.name.set_visible(False)

		# Show Package Icon
		if package.icon_path:
			GLib.idle_add(lambda *_: self.app_icon.set_from_file(package.icon_path))
		else:
			GLib.idle_add(lambda *_: self.app_icon.set_from_icon_name("application-x-executable-symbolic"))

		# Toggle UI Items for Package State
		self.eol_box.set_visible(package.is_eol)
		self.pin_row.set_visible(package.is_runtime)
		self.open_app_button.set_visible(not package.is_runtime)
		self.data_row.set_visible(not package.is_runtime)
		self.uninstall_button.set_sensitive(package_is_not_this_app)
		self.open_app_button.set_sensitive(package_is_not_this_app)
		self.mask_label.set_visible(package.is_masked)
		self.mask_switch.set_active(package.is_masked)
		self.pin_switch.set_active(package.is_pinned)

		# Runtime Specific Logic
		if package.is_runtime:
			self.runtime_row.set_visible(False)
		else:
			has_path = os.path.exists(package.data_path)
			self.trash_data_button.set_sensitive(has_path and package_is_not_this_app)
			self.open_data_button.set_sensitive(has_path)

			if not self.package.dependent_runtime is None:
				self.runtime_row.set_visible(True)
				self.runtime_row.set_subtitle(self.package.dependent_runtime.info["name"])
				self.eol_package_package_status_icon.set_visible(self.package.dependent_runtime.is_eol)

			if has_path:
				self.trash_data_button.set_visible(False)
				self.open_data_button.set_visible(False)
				self.data_spinner.set_visible(True)
				self.data_row.set_subtitle(_("Loading User Data"))

				def callback(size):
					self.trash_data_button.set_visible(True)
					self.open_data_button.set_visible(True)
					self.data_spinner.set_visible(False)
					self.data_row.set_subtitle(size)

				self.package.get_data_size(lambda size: callback(size))
			else:
				self.data_row.set_subtitle(_("No User Data"))
				self.data_spinner.set_visible(False)

		cli_info = None
		try:
			cli_info = package.get_cli_info()
			pkg_description = package.cli_info["description"]
			self.description.set_visible(pkg_description != "")
			self.description.set_label(pkg_description)
		except Exception as e:
			self.toast_overlay.add_toast(ErrorToast(_("Could not get properties"), str(e)).toast)
			return

		failure = None
		for key, row in self.info_rows.items():
			should_be_visible = False

			try:
				row.set_subtitle(cli_info[key])
				should_be_visible = True
			except KeyError:
				if key == "version":
					row.set_subtitle(_("No version information found"))
					should_be_visible = True
			except Exception as e:
				failure = str(e)

			row.set_visible(should_be_visible)

		if not failure is None:
			self.toast_overlay.add_toast(ErrorToast(_("Could not get properties"), str(e)).toast)

		GLib.idle_add(lambda *_: self.stack.set_visible_child(self.nav_view))

		self.more_list.remove_all()
		if self.open_app_button.get_visible():
			if self.package.info["id"] != Config.APP_ID:
				self.more_list.append(self.view_snapshots)
			self.more_list.append(self.copy_launch_command)

		self.more_list.append(self.show_details)

		if self.package.info["id"] != Config.APP_ID:
			self.more_list.append(self.reinstall)

	def open_data_handler(self, *args):
		if error := self.package.open_data():
			self.toast_overlay.add_toast(ErrorToast(_("Could not open data"), str(error)).toast)

	def trash_data_handler(self, *args):
		def on_choice(dialog, response):
			if response != "continue":
				return
			try:
				self.package.trash_data()
				self.set_properties(self.package, refresh=True)
				self.toast_overlay.add_toast(Adw.Toast.new("Trashed User Data"))
				user_data_page = HostInfo.main_window.pages[HostInfo.main_window.user_data_row]
				user_data_page.start_loading()
				user_data_page.end_loading()
				snapshot_list_page = HostInfo.main_window.pages[HostInfo.main_window.snapshots_row].list_page
				snapshot_list_package = snapshot_list_page.package_or_folder
				if not snapshot_list_package is None:
					snapshot_list_page.set_snapshots(snapshot_list_package, True)

			except subprocess.CalledProcessError as cpe:
				self.toast_overlay.add_toast(ErrorToast(_("Could not trash data"), cpe.stderr).toast)
			except Exception as e:
				self.toast_overlay.add_toast(ErrorToast(_("Could not trash data"), str(e)).toast)

		dialog = Adw.AlertDialog(
			heading=_("Send {}'s User Data to the Trash?").format(self.package.info["name"]),
			body=_("Your settings and data for this app will be sent to the trash"),
		)
		dialog.add_response("cancel", _("Cancel"))
		dialog.add_response("continue", _("Trash Data"))
		dialog.connect("response", on_choice)
		dialog.set_response_appearance("continue", Adw.ResponseAppearance.DESTRUCTIVE)
		dialog.present(self.main_window)

	def set_mask_handler(self, *args):
		state = not self.mask_switch.get_active()

		def callback(*args):
			if fail := self.package.failed_mask:
				response = _("Could not Disable Updates") if state else _("Could not Enable Updates")
				fail = fail.stderr if type(fail) is subprocess.CalledProcessError else fail
				self.toast_overlay.add_toast(ErrorToast(response, str(fail)).toast)
				GLib.idle_add(lambda *_: self.mask_switch.set_active(not state))
				GLib.idle_add(lambda *_: self.mask_label.set_visible(not state))
			else:
				response = _("Disabled Updates") if state else _("Enabled Updates")
				self.toast_overlay.add_toast(Adw.Toast(title=response))
				GLib.idle_add(lambda *_: self.mask_switch.set_active(state))
				GLib.idle_add(lambda *_: self.mask_label.set_visible(state))
				self.package.app_row.masked_status_icon.set_visible(state)

		self.package.set_mask(state, callback)

	def set_pin_handler(self, *args):
		state = not self.pin_switch.get_active()

		def callback(*args):
			if fail := self.package.failed_pin:
				response = _("Could not Disable Autoremoval") if state else _("Could not Enable Autoremoval")
				fail = fail.stderr if type(fail) is subprocess.CalledProcessError else fail
				self.toast_overlay.add_toast(ErrorToast(response, str(fail)).toast)
				GLib.idle_add(lambda *_: self.pin_switch.set_active(not state))
			else:
				response = _("Disabled Autoremoval") if state else _("Enabled Autoremoval")
				self.toast_overlay.add_toast(Adw.Toast(title=response))
				GLib.idle_add(lambda *_: self.pin_switch.set_active(state))
				self.package.app_row.pinned_status_icon.set_visible(state)

		self.package.set_pin(state, callback)

	def uninstall_handler(self, *args):
		def on_choice(should_trash):
			self.packages_page.set_status(self.packages_page.uninstalling)
			self.package.uninstall(callback)
			if should_trash:
				try:
					self.package.trash_data()
					self.set_properties(self.package, refresh=True)
				except subprocess.CalledProcessError as cpe:
					self.toast_overlay.add_toast(ErrorToast(_("Could not trash data"), cpe.stderr).toast)
				except Exception as e:
					self.toast_overlay.add_toast(ErrorToast(_("Could not trash data"), str(e)).toast)

		def callback(*args):
			if fail := self.package.failed_uninstall:
				fail = fail.stderr if type(fail) is subprocess.CalledProcessError else fail
				self.toast_overlay.add_toast(ErrorToast(_("Could not uninstall"), str(fail)).toast)
				self.packages_page.set_status(self.packages_page.scrolled_window)
			else:
				self.main_window.refresh_handler()
				HostInfo.main_window.toast_overlay.add_toast(Adw.Toast(title=_("Uninstalled {}").format(self.package.info["name"])))

		dialog = UninstallDialog(on_choice, os.path.exists(self.package.data_path), self.package.info["name"])
		dialog.present(self.main_window)

	def runtime_row_handler(self, *args):
		new_page = self.__class__()
		new_page.packages_page = self.packages_page
		new_page.set_properties(self.package.dependent_runtime)
		self.nav_view.push(new_page)

	def open_app_handler(self, *args):
		self.toast_overlay.add_toast(Adw.Toast(title=_("Opening {}…").format(self.package.info["name"])))

		def callback(*args):
			if fail := self.package.failed_app_run:
				fail = fail.stderr if type(fail) is subprocess.CalledProcessError else fail
				self.toast_overlay.add_toast(ErrorToast(_("Could not open {}").format(self.package.info["name"]), str(fail)).toast)

		self.package.open_app(callback)

	def copy_handler(self, row):
		HostInfo.clipboard.set(row.get_subtitle())
		self.toast_overlay.add_toast(Adw.Toast(title=_("Copied {}").format(row.get_title())))

	def change_version_handler(self, row):
		page = ChangeVersionPage(self.packages_page, self.package)
		self.nav_view.push(page)

	def reinstall_callback(self):
		HostInfo.main_window.refresh_handler()
		if not self.reinstall_did_error:
			HostInfo.main_window.toast_overlay.add_toast(Adw.Toast(title=_("Reinstalled {}").format(self.package.info["name"])))

	def reinstall_error_callback(self, user_facing_label, error_message):
		self.reinstall_did_error = True
		GLib.idle_add(lambda *_: HostInfo.main_window.toast_overlay.add_toast(ErrorToast(user_facing_label, error_message).toast))

	def reinstall_handler(self):
		def on_response(dialog, response):
			if response != "continue":
				return

			self.reinstall_did_error = False
			PackageInstallWorker.install(
				[
					{
						"installation": self.package.info["installation"],
						"remote": self.package.info["origin"],
						"package_names": [self.package.info["ref"]],
						"extra_flags": ["--reinstall"],
					}
				],
				self.packages_page.reinstalling,
				self.reinstall_callback,
				self.reinstall_error_callback,
			)
			self.packages_page.set_status(self.packages_page.reinstalling)

		dialog = Adw.AlertDialog(
			heading=_("Reinstall {}?").format(self.package.info["name"]),
			body=_("This package will be uninstalled, and then reinstalled from the same remote and installation."),
		)
		dialog.add_response("cancel", _("Cancel"))
		dialog.add_response("continue", _("Reinstall"))
		dialog.set_response_appearance("continue", Adw.ResponseAppearance.SUGGESTED)
		dialog.connect("response", on_response)
		dialog.present(HostInfo.main_window)

	def more_menu_handler(self, listbox, row):
		self.more_menu.popdown()
		match row.get_child():
			case self.view_snapshots:
				snapshots_row = HostInfo.main_window.snapshots_row
				snapshots_page = HostInfo.main_window.pages[snapshots_row]
				HostInfo.main_window.activate_row(snapshots_row)
				snapshots_page.show_snapshot(self.package)

			case self.copy_launch_command:
				try:
					HostInfo.clipboard.set(f"flatpak run {self.package.info['ref']}")
					self.toast_overlay.add_toast(Adw.Toast.new(_("Copied launch command")))
				except Exception as e:
					self.toast_overlay.add_toast(ErrorToast(_("Could not copy launch command"), str(e)).toast)

			case self.show_details:
				try:
					Gio.AppInfo.launch_default_for_uri(f"appstream://{self.package.info['id']}", None)
				except Exception as e:
					self.toast_overlay.add_toast(ErrorToast(_("Could not show details"), str(e)).toast)

			case self.reinstall:
				self.reinstall_handler()

	def __init__(self, **kwargs):
		super().__init__(**kwargs)

		# Extra Object Creation
		self.main_window = HostInfo.main_window
		self.info_rows = {
			"version": self.version_row,
			"installed": self.installed_size_row,
			"id": self.id_row,
			"ref": self.ref_row,
			"arch": self.arch_row,
			"branch": self.branch_row,
			"license": self.license_row,
			"sdk": self.sdk_row,
			"origin": self.origin_row,
			"collection": self.collection_row,
			"installation": self.installation_row,
			"commit": self.commit_row,
			"parent": self.parent_row,
			"subject": self.subject_row,
			"date": self.date_row,
		}
		self.loading_tbv.set_content(LoadingStatus(_("Loading Properties"), _("This should only take a moment")))
		self.packages_page = None  # To be set in packages page
		self.__class__.main_window = self.main_window
		self.view_snapshots = Gtk.Label(halign=Gtk.Align.START, label=_("View Snapshots"))
		self.copy_launch_command = Gtk.Label(halign=Gtk.Align.START, label=_("Copy Launch Command"))
		self.show_details = Gtk.Label(halign=Gtk.Align.START, label=_("Show Details"))
		self.reinstall = Gtk.Label(halign=Gtk.Align.START, label=_("Reinstall"))
		self.reinstall_did_error = False

		# Apply

		# Connections
		self.more_list.connect("row-activated", self.more_menu_handler)
		self.open_data_button.connect("clicked", self.open_data_handler)
		self.scrolled_window.get_vadjustment().connect("value-changed", lambda adjustment: self.header_bar.set_show_title(not adjustment.get_value() == 0))
		self.trash_data_button.connect("clicked", self.trash_data_handler)
		self.runtime_row.connect("activated", self.runtime_row_handler)
		self.open_app_button.connect("clicked", self.open_app_handler)
		self.uninstall_button.connect("clicked", self.uninstall_handler)
		self.mask_row.connect("activated", self.set_mask_handler)
		self.pin_row.connect("activated", self.set_pin_handler)
		self.change_version_row.connect("activated", self.change_version_handler)
		for key, row in self.info_rows.items():
			if type(row) is Adw.ActionRow:
				row.connect("activated", self.copy_handler)



from gi.repository import Adw, Gtk, GLib, Gio
from src.host_info import HostInfo
from src.gtk.error_toast import ErrorToast
from src.gtk.attempt_install_dialog import AttemptInstallDialog
import subprocess
import gettext

_ = gettext.gettext


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/user_data_page/data_box.ui")
class DataBox(Gtk.ListBox):
	__gtype_name__ = "DataBox"
	gtc = Gtk.Template.Child

	row = gtc()
	image = gtc()
	title_label = gtc()
	subtitle_label = gtc()
	spinner = gtc()
	size_label = gtc()

	copy_button = gtc()
	open_button = gtc()
	install_button = gtc()
	trash_button = gtc()
	check_button = gtc()

	def human_readable_size(self):
		working_size = self.size
		units = ["KB", "MB", "GB", "TB"]
		# size *= 1024
		for unit in units:
			if working_size < 1024:
				return f"~ {round(working_size)} {unit}"
			working_size /= 1024
		return f"~ {round(working_size)} PB"

	def get_size(self, *args):
		self.size = int(subprocess.run(["du", "-s", self.data_path], capture_output=True, text=True).stdout.split("\t")[0])

	def show_size(self):
		def callback(*args):
			self.size_label.set_label(self.human_readable_size())
			self.spinner.set_visible(False)
			if self.callback:
				self.callback(self.size)

		Gio.Task.new(None, None, callback).run_in_thread(self.get_size)

	def idle_stuff(self):
		self.title_label.set_label(self.title)
		self.subtitle_label.set_label(self.subtitle)
		self.install_button.set_visible(self.is_leftover)
		if self.icon_path:
			self.image.add_css_class("icon-dropshadow")
			self.image.set_from_file(self.icon_path)

	def copy_handler(self, *args):
		try:
			HostInfo.clipboard.set(self.data_path)
			self.toast_overlay.add_toast(Adw.Toast.new(_("Copied data path")))
		except Exception as e:
			self.toast_overlay.add_toast(ErrorToast(_("Could not copy data path"), str(e)).toast)

	def open_handler(self, *args):
		try:
			Gio.AppInfo.launch_default_for_uri(f"file://{self.data_path}", None)
			self.toast_overlay.add_toast(Adw.Toast.new(_("Opened data folder")))
		except GLib.GError as e:
			self.toast_overlay.add_toast(ErrorToast(_("Could not open folder"), str(e)).toast)

	def install_handler(self, *args):
		self.parent_page.should_rclick = False

		def why_cant_this_just_be_a_lambda(*args):
			self.parent_page.should_rclick = True

		AttemptInstallDialog([self.subtitle], why_cant_this_just_be_a_lambda)

	def trash_handler(self, *args):
		self.failed_trash = False

		def thread(*args):
			try:
				subprocess.run(["gio", "trash", self.data_path], check=True, text=True, capture_output=True)
				properties_page = HostInfo.main_window.pages[HostInfo.main_window.packages_row].properties_page
				properties_package = properties_page.package
				if not properties_package is None:
					properties_page.set_properties(properties_package, True)

				snapshot_list_page = HostInfo.main_window.pages[HostInfo.main_window.snapshots_row].list_page
				snapshot_list_package = snapshot_list_page.package_or_folder
				if not snapshot_list_package is None:
					snapshot_list_page.set_snapshots(snapshot_list_package, True)

			except subprocess.CalledProcessError as cpe:
				self.failed_trash = cpe.stderr
			except Exception as e:
				self.failed_trash = e

		def callback(*args):
			if self.failed_trash:
				self.toast_overlay.add_toast(ErrorToast(_("Could not trash data"), str(self.failed_trash)).toast)
			else:
				self.toast_overlay.add_toast(Adw.Toast.new("Trashed data"))
				if self.trash_callback:
					self.trash_callback(self)

		def on_response(_, response):
			self.parent_page.should_rclick = True
			if response != "continue":
				return

			Gio.Task.new(None, None, callback).run_in_thread(thread)

		self.parent_page.should_rclick = False
		dialog = Adw.AlertDialog(heading=_("Trash {}'s Data?").format(self.title), body=_("{}'s data will be sent to the trash").format(self.title))
		dialog.add_response("cancel", _("Cancel"))
		dialog.add_response("continue", _("Continue"))
		dialog.set_response_appearance("continue", Adw.ResponseAppearance.DESTRUCTIVE)
		dialog.connect("response", on_response)
		dialog.present(HostInfo.main_window)

	def __init__(self, parent_page, toast_overlay, is_leftover, title, subtitle, data_path, icon_path=None, callback=None, trash_callback=None, **kwargs):
		super().__init__(**kwargs)

		# Extra Object Creation
		self.parent_page = parent_page
		self.toast_overlay = toast_overlay
		self.is_leftover = is_leftover
		self.title = title
		self.subtitle = subtitle
		self.icon_path = icon_path
		self.data_path = data_path
		self.callback = callback
		self.trash_callback = trash_callback
		self.size = None
		self.failed_trash = None

		# Apply
		self.idle_stuff()
		self.show_size()
		if subtitle == "io.github.flattool.Warehouse":
			self.check_button.set_active = lambda *_: None
			self.check_button.set_sensitive(False)
			self.trash_button.set_sensitive(False)

		# Connections
		self.copy_button.connect("clicked", self.copy_handler)
		self.open_button.connect("clicked", self.open_handler)
		self.install_button.connect("clicked", self.install_handler)
		self.trash_button.connect("clicked", self.trash_handler)



from gi.repository import Adw, Gtk
from src.host_info import HostInfo
import gettext

_ = gettext.gettext


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/gtk/installation_chooser.ui")
class InstallationChooser(Adw.PreferencesGroup):
	__gtype_name__ = "InstallationChooser"
	gtc = Gtk.Template.Child

	user_row = gtc()
	system_row = gtc()
	single_row = gtc()
	choice_row = gtc()
	user_check = gtc()
	system_check = gtc()
	single_check = gtc()
	choice_check = gtc()

	def get_installation(self):
		for button, func in self.check_buttons.items():
			if button.get_active():
				return func()

		return ""  # Case for when no button is active (which shouldn't happen)

	def set_content_strings(self, content_name, is_plural):
		if is_plural:
			self.user_row.set_subtitle(_("These {} will only be available to you").format(content_name))
			self.system_row.set_subtitle(_("These {} will be available to everyone").format(content_name))
			self.set_description(_("Choose how these {} will be installed").format(content_name))
		else:
			self.user_row.set_subtitle(_("This {} will only be available to you").format(content_name))
			self.system_row.set_subtitle(_("This {} will be available to everyone").format(content_name))
			self.set_description(_("Choose how this {} will be installed").format(content_name))

	def __init__(self, **kwargs):
		super().__init__(**kwargs)

		self.check_buttons = {
			self.user_check: lambda: "user",
			self.system_check: lambda: "system",
			self.single_check: self.single_row.get_title,
			self.choice_check: lambda: self.choice_row.get_selected_item().get_string(),
		}

		# Apply
		custom_installations = []
		for installation in HostInfo.installations:
			if installation.startswith("user") or installation.startswith("system"):
				continue

			custom_installations.append(installation)

		if len(custom_installations) == 1:
			self.single_row.set_visible(True)
			self.single_row.set_title(custom_installations[0])
		elif len(custom_installations) > 1:
			self.choice_row.set_visible(True)
			self.choice_row.set_model(Gtk.StringList(strings=custom_installations))

		# Connections
		self.choice_row.connect("notify::css-classes", lambda *_: self.choice_check.activate())



from gi.repository import Adw, Gtk
import gettext

_ = gettext.gettext


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/install_page/file_install_dialog.ui")
class FileInstallDialog(Adw.Dialog):
	__gtype_name__ = "FileInstallDialog"
	gtc = Gtk.Template.Child

	packages_group = gtc()
	installation_chooser = gtc()
	cancel_button = gtc()
	apply_button = gtc()

	def generate_list(self):
		for file in self.files:
			row = Adw.ActionRow(title=file.get_basename())
			row.add_prefix(Gtk.Image(icon_name="warehouse:flatpak-symbolic"))
			self.packages_group.add(row)

	def on_response(self, *args):
		self.on_add(self.installation_chooser.get_installation(), self.files)
		self.close()

	def __init__(self, parent_page, files, on_add, **kwargs):
		super().__init__(**kwargs)

		# Extra Object Creation
		self.parent_page = parent_page
		self.files = files
		self.on_add = on_add

		# Apply
		self.generate_list()
		if len(files) > 1:
			self.set_title(_("Install Packages"))
			# self.packages_group.set_title(_("Review Packages"))
			self.packages_group.set_description(_("The following packages will be installed"))
			self.installation_chooser.set_content_strings(_("Packages"), True)
		else:
			self.set_title(_("Install a Package"))
			# self.packages_group.set_title(_("Review Package"))
			self.packages_group.set_description(_("The following package will be installed"))
			self.installation_chooser.set_content_strings(_("package"), False)

		# Connections
		self.cancel_button.connect("clicked", lambda *_: self.close())
		self.apply_button.connect("clicked", self.on_response)



from gi.repository import Adw, Gtk, GLib
from src.host_info import HostInfo
from src.install_page.select_page import SelectPage
from src.install_page.pending_page import PendingPage
from src.gtk.sidebar_button import SidebarButton
from src.gtk.loading_status import LoadingStatus
from src.package_install_worker import PackageInstallWorker
from src.gtk.error_toast import ErrorToast
import gettext

_ = gettext.gettext


@Gtk.Template(resource_path="/io/github/flattool/Warehouse/install_page/install_page.ui")
class InstallPage(Adw.BreakpointBin):
	__gtype_name__ = "InstallPage"
	gtc = Gtk.Template.Child

	break_point = gtc()
	split_view = gtc()
	multi_view = gtc()
	select_page = gtc()
	pending_page = gtc()
	status_stack = gtc()
	loading_view = gtc()
	installing_view = gtc()
	bottom_sheet = gtc()
	bottom_child = gtc()
	bottom_label = gtc()

	# Referred to in the main window
	# It is used to determine if a new page should be made or not
	# This must be set to the created object from within the class's __init__ method
	instance = None
	page_name = "install"

	current_installation = ""
	current_remote = None
	did_error = False

	def start_loading(self):
		self.total_added_packages = 0
		self.bottom_bar_visual_handler(False)
		self.status_stack.set_visible_child(self.loading_view)
		self.select_page.start_loading()
		self.pending_page.reset()

	def end_loading(self):
		self.select_page.end_loading()
		self.status_stack.set_visible_child(self.multi_view)

	def install_callback(self):
		HostInfo.main_window.refresh_handler()
		if self.did_error:
			return
		if not PackageInstallWorker.cancelled:
			HostInfo.main_window.toast_overlay.add_toast(Adw.Toast(title=_("Installed Packages")))

	def install_error_callback(self, user_facing_label, error_message):
		self.did_error = True
		GLib.idle_add(lambda *_: HostInfo.main_window.toast_overlay.add_toast(ErrorToast(user_facing_label, error_message).toast))

	def install_packages(self, package_requests):
		self.did_error = False
		if PackageInstallWorker.install(package_requests, self.installing_status, self.install_callback, self.install_error_callback):
			self.status_stack.set_visible_child(self.installing_view)

	def bottom_bar_visual_handler(self, is_added):
		total = self.total_added_packages
		if total == 0:
			self.bottom_child.set_reveal_child(False)
			self.bottom_sheet.set_bottom_bar(None)
		elif total == 1:
			self.bottom_label.set_label(_("{} Pending Package").format(1))
			if is_added:
				self.bottom_sheet.set_bottom_bar(self.bottom_child)
				self.bottom_child.set_reveal_child(True)
		else:
			self.bottom_label.set_label(_("{} Pending Packages").format(total))

	def package_added(self):
		self.total_added_packages += 1
		self.bottom_bar_visual_handler(True)

	def package_removed(self):
		self.total_added_packages -= 1
		self.bottom_bar_visual_handler(False)

	def __init__(self, main_window, **kwargs):
		super().__init__(**kwargs)
		self.instance = self

		# Extra Object Creation
		self.installing_status = LoadingStatus(_("Installing Packages"), _("This could take a while"), True, PackageInstallWorker.cancel)
		self.total_added_packages = 0

		# Connections

		# Apply
		self.select_page.results_page.pending_page = self.pending_page
		self.select_page.results_page.install_page = self
		self.loading_view.set_content(LoadingStatus(_("Loading Installation Options"), _("This should only take a moment")))
		self.installing_view.set_content(self.installing_status)
